//---------------------------------------------------------------------------//
//                                                                           //
//  Source file: RC4.java                                                    //
//                                                                           //
//  Description: Implements RC4 stream cipher algorithm base class           //
//                                                                           //
//  Author: Pavel Hrdina, paul@compelson.com                                 //
//                                                                           //
//---------------------------------------------------------------------------//

import java.io.Serializable;
    
//
//  Class implementing RC4 stream cipher algorithm
//

public class RC4 implements Serializable
{
    protected short[] KeyState;
    protected short[] State;
    protected int X, Y;

    //
    //  Constructor
    //

    public RC4(byte[] Key)
    {
        int KeyIndex = 0;
        int Index2 = 0;
        int Counter;

        //
        //  Allocating and initializing array for KeyState
        //

        KeyState = new short[256];

        for (Counter = 0; Counter < KeyState.length; Counter++)
        {
            KeyState[Counter] = (short)Counter;
        }

        //
        //  Modifying KeyState based on the Key
        //

        for (Counter = 0; Counter < KeyState.length; Counter++)
        {
            short Temp;
            int KeyElement;

            //
            //  Get one byte from key converting it to int (0-255)
            //

            KeyElement = (Key[KeyIndex] < 0) ? Key[KeyIndex] + 256
                                             : Key[KeyIndex];

            //
            //  Counting new Index2 value (wrapping around KeyState)
            //

            Index2 = (KeyElement + KeyState[Counter] + Index2) % KeyState.length;

            //
            //  Swap values at KeyState[Counter] and KeyState[Index2]
            //

            Temp = KeyState[Counter];
            KeyState[Counter] = KeyState[Index2];
            KeyState[Index2] = Temp;

            //
            //  Selecting next byte from the Key (wrapping around it)
            //

            KeyIndex = (KeyIndex + 1) % Key.length;
        }
    }

    //
    //  Initialize one encryption
    //

    public void Initialize()
    {
        int Counter;

        //
        //  Initializing instance X and Y variables
        //

        X = 0;
        Y = 0;

        //
        //  Allocating and initializing array for State
        //

        State = new short[KeyState.length];

        for (Counter = 0; Counter < State.length; Counter++)
        {
            State[Counter] = KeyState[Counter];
        }
    }

    //
    //  Terminate one encryption
    //

    public void Finalize()
    {
        State = null;
    }

    //
    //  Physically does encryption and decryption
    //

    public void Update(byte[] Input, byte[] Output)
    {
        int X, Y;
        int Counter;

        //
        //  Get instance X and Y to local X and Y
        //

        X = this.X;
        Y = this.Y;

        //
        //  Loop for every input byte
        //

        for (Counter=0; Counter<Input.length; Counter++)
        {
            int XorIndex;
            short Temp;

            //
            //  Counting new X and Y values
            //

            X = (X + 1) % State.length;
            Y = (State[X] + Y) % State.length;

            //
            //  Swap values at State[X] and State[Y]
            //

            Temp = State[X];
            State[X] = State[Y];
            State[Y] = Temp;

            //
            //  Counting XorIndex
            //

            XorIndex = (State[X] + State[Y]) % State.length;

            //
            //  Encrypting/Decrypting current byte
            //

            Output[Counter] = (byte)(Input[Counter] ^ (byte)State[XorIndex]);
        }

        //
        //  Save new X and Y values to instance X and Y
        //

        this.X = X;
        this.Y = Y;
    }
}
